diff --git a/pgadmin/dlg/dlgServer.cpp b/pgadmin/dlg/dlgServer.cpp
index e7a9741..4ffb02f 100644
--- a/pgadmin/dlg/dlgServer.cpp
+++ b/pgadmin/dlg/dlgServer.cpp
@@ -43,6 +43,7 @@
 #define txtPassword     CTRL_TEXT("txtPassword")
 #define txtDbRestriction CTRL_TEXT("txtDbRestriction")
 #define colourPicker    CTRL_COLOURPICKER("colourPicker")
+#define cbGroup         CTRL_COMBOBOX("cbGroup")
 
 
 BEGIN_EVENT_TABLE(dlgServer, dlgProperty)
@@ -59,6 +60,8 @@ BEGIN_EVENT_TABLE(dlgServer, dlgProperty)
     EVT_CHECKBOX(XRCID("chkRestore"),               dlgProperty::OnChange)
     EVT_CHECKBOX(XRCID("chkTryConnect"),            dlgServer::OnChangeTryConnect)
     EVT_COLOURPICKER_CHANGED(XRCID("colourPicker"), dlgServer::OnChangeColour)
+    EVT_TEXT(XRCID("cbGroup"),                      dlgProperty::OnChange)
+    EVT_COMBOBOX(XRCID("cbGroup"),                  dlgProperty::OnChange)
     EVT_BUTTON(wxID_OK,                             dlgServer::OnOK)
 END_EVENT_TABLE();
 
@@ -155,6 +158,7 @@ void dlgServer::OnOK(wxCommandEvent &ev)
         wxColour colour = colourPicker->GetColour();
         wxString sColour = colour.GetAsString(wxC2S_HTML_SYNTAX);
         server->iSetColour(sColour);
+        server->iSetGroup(cbGroup->GetValue());
         mainForm->execSelChange(server->GetId(), true);
         mainForm->GetBrowser()->SetItemText(item, server->GetFullName());
 
@@ -254,6 +258,7 @@ int dlgServer::Go(bool modal)
         chkRestore->SetValue(server->GetRestore());
         txtDbRestriction->SetValue(server->GetDbRestriction());
         colourPicker->SetColour(server->GetColour());
+        cbGroup->SetValue(server->GetGroup());
 
         stPassword->Disable();
         txtPassword->Disable();
@@ -338,7 +343,8 @@ void dlgServer::CheckChange()
                || chkStorePwd->GetValue() != server->GetStorePwd()
                || chkRestore->GetValue() != server->GetRestore()
                || txtDbRestriction->GetValue() != server->GetDbRestriction()
-               || sColour != sColour2;
+               || sColour != sColour2
+               || cbGroup->GetValue() != server->GetGroup();
     }
 
 
diff --git a/pgadmin/frm/frmMain.cpp b/pgadmin/frm/frmMain.cpp
index 67e9b67..d638f77 100644
--- a/pgadmin/frm/frmMain.cpp
+++ b/pgadmin/frm/frmMain.cpp
@@ -1129,43 +1129,89 @@ void frmMain::StoreServers()
     gethostname(buf, 255); 
     wxString hostname = wxString(buf, wxConvUTF8);
 
-    // Write the individual servers
-    // Iterate through all the child nodes of the Servers node
-    treeObjectIterator servers(browser, serversObj);
-
-    while ((server = (pgServer*)servers.GetNextObject()) != 0)
+    wxTreeItemIdValue itemcookie;
+    wxTreeItemId item = browser->GetFirstChild(browser->GetRootItem(), itemcookie);
+    while (item)
     {
-        if (server->IsCreatedBy(serverFactory))
+        if (browser->ItemHasChildren(item))
         {
-            wxString key;
-            ++numServers;
-
-            key.Printf(wxT("Servers/%d/"), numServers);
-            settings->Write(key + wxT("Server"), server->GetName());
-            settings->Write(key + wxT("Description"), server->GetDescription());
-            settings->Write(key + wxT("ServiceID"), server->GetServiceID());
-            settings->Write(key + wxT("DiscoveryID"), server->GetDiscoveryID());
-            settings->Write(key + wxT("Port"), server->GetPort());
-            settings->Write(key + wxT("StorePwd"), server->GetStorePwd());
-            settings->Write(key + wxT("Restore"), server->GetRestore());
-            settings->Write(key + wxT("Database"), server->GetDatabaseName());
-            settings->Write(key + wxT("Username"), server->GetUsername());
-            settings->Write(key + wxT("LastDatabase"), server->GetLastDatabase());
-            settings->Write(key + wxT("LastSchema"), server->GetLastSchema());
-            settings->Write(key + wxT("DbRestriction"), server->GetDbRestriction());
-            settings->Write(key + wxT("Colour"), server->GetColour());
-            settings->Write(key + wxT("SSL"), server->GetSSL());
-
-            pgCollection *coll=browser->FindCollection(databaseFactory, server->GetId());
-            if (coll)
+            wxTreeItemIdValue itemcookie2;
+            wxTreeItemId item2 = browser->GetFirstChild(item, itemcookie2);
+            while (item2)
             {
-                treeObjectIterator dbs(browser, coll);
-                pgDatabase *db;
+                server = (pgServer*)browser->GetItemData(item2);
+                if (server->IsCreatedBy(serverFactory))
+                {
+                    wxString key;
+                    ++numServers;
+
+                    key.Printf(wxT("Servers/%d/"), numServers);
+                    settings->Write(key + wxT("Server"), server->GetName());
+                    settings->Write(key + wxT("Description"), server->GetDescription());
+                    settings->Write(key + wxT("ServiceID"), server->GetServiceID());
+                    settings->Write(key + wxT("DiscoveryID"), server->GetDiscoveryID());
+                    settings->Write(key + wxT("Port"), server->GetPort());
+                    settings->Write(key + wxT("StorePwd"), server->GetStorePwd());
+                    settings->Write(key + wxT("Restore"), server->GetRestore());
+                    settings->Write(key + wxT("Database"), server->GetDatabaseName());
+                    settings->Write(key + wxT("Username"), server->GetUsername());
+                    settings->Write(key + wxT("LastDatabase"), server->GetLastDatabase());
+                    settings->Write(key + wxT("LastSchema"), server->GetLastSchema());
+                    settings->Write(key + wxT("DbRestriction"), server->GetDbRestriction());
+                    settings->Write(key + wxT("Colour"), server->GetColour());
+                    settings->Write(key + wxT("SSL"), server->GetSSL());
+                    settings->Write(key + wxT("Group"), server->GetGroup());
+
+                    pgCollection *coll=browser->FindCollection(databaseFactory, server->GetId());
+                    if (coll)
+                    {
+                        treeObjectIterator dbs(browser, coll);
+                        pgDatabase *db;
 
-                while ((db=(pgDatabase*)dbs.GetNextObject()) != 0)
-                    settings->Write(key + wxT("Databases/") + db->GetName() + wxT("/SchemaRestriction"), db->GetSchemaRestriction());
+                        while ((db=(pgDatabase*)dbs.GetNextObject()) != 0)
+                            settings->Write(key + wxT("Databases/") + db->GetName() + wxT("/SchemaRestriction"), db->GetSchemaRestriction());
+                    }
+                }
+                item2 = browser->GetNextChild(item, itemcookie2);
             }
         }
+        else
+        {
+            server = (pgServer*)browser->GetItemData(item);
+            if (server->IsCreatedBy(serverFactory))
+            {
+                wxString key;
+                ++numServers;
+
+                key.Printf(wxT("Servers/%d/"), numServers);
+                settings->Write(key + wxT("Server"), server->GetName());
+                settings->Write(key + wxT("Description"), server->GetDescription());
+                settings->Write(key + wxT("ServiceID"), server->GetServiceID());
+                settings->Write(key + wxT("DiscoveryID"), server->GetDiscoveryID());
+                settings->Write(key + wxT("Port"), server->GetPort());
+                settings->Write(key + wxT("StorePwd"), server->GetStorePwd());
+                settings->Write(key + wxT("Restore"), server->GetRestore());
+                settings->Write(key + wxT("Database"), server->GetDatabaseName());
+                settings->Write(key + wxT("Username"), server->GetUsername());
+                settings->Write(key + wxT("LastDatabase"), server->GetLastDatabase());
+                settings->Write(key + wxT("LastSchema"), server->GetLastSchema());
+                settings->Write(key + wxT("DbRestriction"), server->GetDbRestriction());
+                settings->Write(key + wxT("Colour"), server->GetColour());
+                settings->Write(key + wxT("SSL"), server->GetSSL());
+                settings->Write(key + wxT("Group"), server->GetGroup());
+
+                pgCollection *coll=browser->FindCollection(databaseFactory, server->GetId());
+                if (coll)
+                {
+                    treeObjectIterator dbs(browser, coll);
+                    pgDatabase *db;
+
+                    while ((db=(pgDatabase*)dbs.GetNextObject()) != 0)
+                        settings->Write(key + wxT("Databases/") + db->GetName() + wxT("/SchemaRestriction"), db->GetSchemaRestriction());
+                }
+            }
+        }
+        item = browser->GetNextChild(browser->GetRootItem(), itemcookie);
     }
 
     // Write the server count
@@ -1181,9 +1227,23 @@ void frmMain::RetrieveServers()
 
     serverFactory.CreateObjects(serversObj, browser);
     
+    // Count number of servers
+    int total = 0;
+    wxTreeItemIdValue itemcookie;
+    wxTreeItemId item = browser->GetFirstChild(browser->GetRootItem(), itemcookie);
+    while (item)
+    {
+        wxLogError(browser->GetItemText(item));
+        if (browser->ItemHasChildren(item))
+            total += browser->GetChildrenCount(item, false);
+        else
+            total += 1;
+        item = browser->GetNextChild(browser->GetRootItem(), itemcookie);
+    }
+
     // Reset the Servers node text
     wxString label;
-    label.Printf(_("Servers (%d)"), browser->GetChildrenCount(serversObj->GetId(), false));
+    label.Printf(_("Servers (%d)"), total);
     browser->SetItemText(serversObj->GetId(), label);
 }
 
diff --git a/pgadmin/include/schema/pgServer.h b/pgadmin/include/schema/pgServer.h
index 9567897..0173afd 100644
--- a/pgadmin/include/schema/pgServer.h
+++ b/pgadmin/include/schema/pgServer.h
@@ -122,6 +122,9 @@ public:
 	void iSetColour(const wxString &s) { colour = s; }
 	wxString GetColour() { return colour; }
 
+	void iSetGroup(const wxString &s) { group = s; }
+	wxString GetGroup() { return group; }
+
     bool HasPrivilege(const wxString &objTyp, const wxString &objName, const wxString &priv) { return conn->HasPrivilege(objTyp, objName, priv); }
     bool ExecuteVoid(const wxString& sql) { return conn->ExecuteVoid(sql); }
     wxString ExecuteScalar(const wxString& sql) { return conn->ExecuteScalar(sql); }
@@ -162,6 +165,7 @@ private:
     wxString versionNum;
     wxString dbRestriction;
 	wxString colour;
+	wxString group;
 
     bool inRecovery;
     wxString receiveLoc, replayLoc;
diff --git a/pgadmin/schema/pgServer.cpp b/pgadmin/schema/pgServer.cpp
index e62f552..1eeb0a8 100644
--- a/pgadmin/schema/pgServer.cpp
+++ b/pgadmin/schema/pgServer.cpp
@@ -1107,10 +1107,14 @@ bool pgServerObjCollection::CanCreate()
 
 pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, const wxString &restr)
 {
+    wxTreeItemId folder, itm;
+    wxTreeItemIdValue foldercookie;
+    bool found;
+
     long numServers=settings->Read(wxT("Servers/Count"), 0L);
 
     long loop, port, ssl=0;
-    wxString key, servername, description, database, username, lastDatabase, lastSchema, storePwd, restore, serviceID, discoveryID, dbRestriction, colour;
+    wxString key, servername, description, database, username, lastDatabase, lastSchema, storePwd, restore, serviceID, discoveryID, dbRestriction, colour, group;
     pgServer *server=0;
 
     wxArrayString discoveredServers;
@@ -1120,6 +1124,7 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
     gethostname(buf, 255); 
     wxString hostname = wxString(buf, wxConvUTF8);
 
+    //wxLogError(wxT("Loading previously registered servers"));
     wxLogInfo(wxT("Loading previously registered servers"));
 
     for (loop = 1; loop <= numServers; ++loop)
@@ -1139,6 +1144,9 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
         settings->Read(key + wxT("LastSchema"), &lastSchema, wxEmptyString);
         settings->Read(key + wxT("DbRestriction"), &dbRestriction, wxEmptyString);
         settings->Read(key + wxT("Colour"), &colour, wxEmptyString);
+        settings->Read(key + wxT("Group"), &group, wxEmptyString);
+
+        //wxLogError(servername);
 
         // Sanitize the colour
         colour = colour.Trim();
@@ -1167,9 +1175,36 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
         server->iSetDiscovered(false);
         server->iSetDbRestriction(dbRestriction);
         server->iSetColour(colour);
+        server->iSetGroup(group);
         server->iSetServerIndex(loop);
-        wxTreeItemId itm = browser->AppendItem(obj->GetId(), server->GetFullName(), server->GetIconId(), -1, server);
-        browser->SortChildren(obj->GetId());
+
+        if (group.IsEmpty())
+        {
+            folder = obj->GetId();
+        }
+        else
+        {
+            found = false;
+            if (browser->ItemHasChildren(obj->GetId()))
+            {
+                folder = browser->GetFirstChild(obj->GetId(), foldercookie);
+                while (!found && folder)
+                {
+                    if (browser->GetItemText(folder) == group)
+                        found = true;
+                    else
+                        folder = browser->GetNextChild(obj->GetId(), foldercookie);
+                }
+            }
+
+            if (!found)
+            {
+                folder = browser->AppendItem(obj->GetId(), group, obj->GetIconId());
+            }
+        }
+
+        itm = browser->AppendItem(folder, server->GetFullName(), server->GetIconId(), -1, server);
+        browser->SortChildren(folder);
         if (!server->GetColour().IsEmpty())
             browser->SetItemBackgroundColour(itm, wxColour(server->GetColour()));
 
diff --git a/pgadmin/ui/dlgServer.xrc b/pgadmin/ui/dlgServer.xrc
index e534149..10af760 100644
--- a/pgadmin/ui/dlgServer.xrc
+++ b/pgadmin/ui/dlgServer.xrc
@@ -195,6 +195,21 @@
                   <flag>wxEXPAND|wxALIGN_CENTRE_VERTICAL|wxALL</flag>
                   <border>4</border>
                 </object>
+                <object class="sizeritem">
+                  <object class="wxStaticText" name="stGroup">
+                    <label>Group</label>
+                  </object>
+                  <flag>wxALIGN_CENTRE_VERTICAL|wxTOP|wxLEFT|wxRIGHT</flag>
+                  <border>4</border>
+                </object>
+                <object class="sizeritem">
+                  <object class="wxComboBox" name="cbGroup">
+                    <style>wxCB_DROPDOWN</style>
+                    <content/>
+                  </object>
+                  <flag>wxEXPAND|wxALIGN_CENTRE_VERTICAL|wxTOP|wxLEFT|wxRIGHT</flag>
+                  <border>4</border>
+                </object>
               </object>
             </object>
           </object>
